#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       communication.py
#
#       Copyright 2010 Jelle Smet <web@smetj.net>
#
#       This file is part of Monitoring python library.
#
#           Monitoring python library is free software: you can redistribute it and/or modify
#           it under the terms of the GNU General Public License as published by
#           the Free Software Foundation, either version 3 of the License, or
#           (at your option) any later version.
#
#           Monitoring python library is distributed in the hope that it will be useful,
#           but WITHOUT ANY WARRANTY; without even the implied warranty of
#           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#           GNU General Public License for more details.
#
#           You should have received a copy of the GNU General Public License
#           along with Monitoring python library.  If not, see <http://www.gnu.org/licenses/>.
import socket
import os
import threading
import time
import random
import Queue
from stat import ST_MODE,S_ISFIFO
from urllib import urlencode
import urllib2
from xml.etree import ElementTree
from hashlib import md5
import sys

class LoadBalance():
    '''Accepts a list of destinations (ip/hostnames) and chooses based upon self.style an ip address.
    There is no sanitiy checking done on the input.  You could provide a list of dog names if you want.
    The style of selection is defined at init.'''
    def __init__(self,style='random'):
        '''By default a randon style selection is done. '''
        self.style=style
        self.previous_destination=None
    def choose(self,destinations):
        '''Accepts a list of destinations and returns a value from it based upon the style chosen on init.'''
        if self.style=='random':
            return destinations[random.randint(0,len(destinations)-1)]
        elif self.style=='next':
            if self.previous_destination == None or self.previous_destination == len(destinations)-1:
                self.previous_destination=0
            else:
                self.previous_destination+=1
            return destinations[self.previous_destination]
        elif self.style=='first':
            return destinations[0]
class Lookup():
    '''Accepts a string, checks if it's a valid ip address, does a reverse lookup and returns the hostname.  If this fails, the original input is returned.'''
    def __init__(self):
        pass
    def reverse(self,ip=None):
        try:
            socket.inet_aton(ip)
            hostname = socket.gethostbyaddr(ip)
            return hostname.lower()
        except socket.error:
            return ip.lower()
class SubmitListener(threading.Thread):
    '''A thread which monitors the input_queue for data and submits this to the correct OutputQueue, and if that doesn't exist, it creates a new OutputQueue object
    The format of the packages to dump into self.input_queue:
    {"destination":{
        "type":"file",
        "locations":[ "/tmp/moncli_output1.txt","/tmp/moncli_output2.txt" ],
        "username":"default",
        "password":"changeme"
        "token":"fubar"},
    "requestUUID":uuid
    "data":"what to write to command pipe"
      }'''

    def __init__(self,timeout=60,retries=5,chunks=1000,quota=0,logger=None,blockcallback=None):
        threading.Thread.__init__(self)
        self.timeout=timeout
        self.retries=retries
        self.chunks=chunks
        self.quota=quota
        self.logger=logger
        self.loop=blockcallback
        self.input_queue=Queue.Queue(0)
        self.output_queues={}
        self.logger.info("SubmitListener thread started.")
        self.daemon=True
        self.start()
    def run(self):
        while self.loop.block() == True:
            while not self.input_queue.empty():
                package=self.input_queue.get()
                if package['external_command'] == None or package['external_command'] == '':
                    self.logger.error("Request %s produced no output.  It's purged."%(package.get('requestUUID','?')))
                else:
                    self.__submit(package)
            time.sleep(0.1)
        self.logger.info("SubmitListener thread stopped.")
    def dump(self,package):
        '''Using dump, the average package size is calculated prior to submitting it to the Queue.  This is the preferred way for writing data ino SubmitListener.'''
        queue_name = self.__queue_name(package['destination'])

        #If we don't have a queue like that create a new one
        if not self.output_queues.has_key(queue_name):
            self.output_queues[queue_name] = OutputQueue(   name=queue_name,
                                    destination=package['destination'],
                                    timeout=self.timeout,
                                    retries=self.retries,
                                    chunks=self.chunks,
                                    logger=self.logger,
                                    blockcallback=self.loop)

        if self.quota > 0:
            if self.output_queues[queue_name].avg_data_size * self.output_queues[queue_name].queue.qsize() < self.quota:
                #Keep track of the average data size
                self.output_queues[queue_name].avg_data_size=(self.output_queues[queue_name].avg_data_size+int(sys.getsizeof(package['external_command'])))/2
                self.output_queues[queue_name].queue.put(package['external_command'])
            else:
                raise Exception('queue oversized')
        else:
            self.output_queues[queue_name].avg_data_size=(self.output_queues[queue_name].avg_data_size+int(sys.getsizeof(package['external_command'])))/2
            self.output_queues[queue_name].queue.put(package['external_command'])


    def __submit(self,package):
        #Check if queue exists
        #if a queue with such a destination doesn't exist anymore then create a new one
        queue_name = self.__queue_name(package['destination'])
        if not self.output_queues.has_key(queue_name):
            self.output_queues[queue_name] = OutputQueue(   name=queue_name,
                                    destination=package['destination'],
                                    timeout=self.timeout,
                                    retries=self.retries,
                                    chunks=self.chunks,
                                    logger=self.logger,
                                    blockcallback=self.loop)
        #Keep track of the average data size
        self.output_queues[queue_name].avg_data_size=(self.output_queues[queue_name].avg_data_size+int(sys.getsizeof(package['external_command'])))/2
        self.output_queues[queue_name].queue.put(package['external_command'])
    def __queue_name(self,destination):
        return md5(str(destination)).hexdigest()
class OutputQueue(threading.Thread):
    def __init__(self,name=None,destination=None,timeout=None,retries=None,chunks=None,logger=None,blockcallback=None):
        threading.Thread.__init__(self)
        self.name=name
        self.type=destination['type']
        self.locations=destination['locations']
        self.username=destination.get('username',None)
        self.password=destination.get('password',None)
        self.token=destination.get('token',None)
        self.timeout=timeout
        self.retries=retries
        self.chunks=chunks
        self.logger=logger
        self.loop=blockcallback
        self.queue=Queue.Queue(0)
        self.loadbalance=LoadBalance()
        self.avg_data_size=0
        self.queue_size=0
        self.daemon=True
        self.submitlock=False
        self.submittimer=1
        self.submitmaxtimer=3600
        self.start()
    def run(self):
        self.logger.info('Delivery queue %s of type %s started with destination %s.'%(self.name,self.type,self.locations))
        while self.loop.block() == True:
            bulk = []
            while not self.queue.empty() and len(bulk) <= self.chunks:
                bulk.append(self.queue.get())
            if len(bulk) > 0 and self.submitlock != True:
                while self.__submit(    type=self.type,
                            locations=self.locations,
                            data=bulk,
                            size=len(bulk),
                            queue_size=self.queue.qsize(),
                            queue_bytes=self.get_size()
                            ) != True:
                    self.logger.info('Setting submitlock and wait for %s seconds.'%(self.submittimer))
                    self.submitlock=True
                    time.sleep(self.submittimer)
                    #increment sleeper with random timer
                    if  self.submittimer < self.submitmaxtimer:
                        self.submittimer = random.randint(self.submittimer, self.submittimer*2)
            self.submitlock=False
            self.submittimer=1
            time.sleep(0.5)
    def __submit(self,type=None,locations=None,data=None,size=None,queue_size=None,queue_bytes=None):
        location = self.loadbalance.choose(locations)
        if type == 'file':
            connector = DeliverFile(    location=location,
                            data=data,
                            size=size,
                            queue_size=queue_size,
                            queue_bytes=queue_bytes,
                            logger=self.logger)
        elif type == 'named pipe' or type == 'local' or type == 'pipe':
            connector = DeliverNamedPipe(   location=location,
                            data=data,
                            size=size,
                            queue_size=queue_size,
                            queue_bytes=queue_bytes,
                            logger=self.logger)
        elif type == 'nscaweb':
            connector = DeliverNscaweb( location=location,
                            data=data,
                            username=self.username,
                            password=self.password,
                            size=size,
                            queue_size=queue_size,
                            queue_bytes=queue_bytes,
                            logger=self.logger)
        elif type == 'nrdp':
            connector = DeliverNrdp(    location=location,
                            data=data,
                            username=self.username,
                            password=self.password,
                            token=self.token,
                            size=size,
                            queue_size=queue_size,
                            queue_bytes=queue_bytes,
                            logger=self.logger)
        connector.join(30)
        if connector.isAlive() == True:
            self.logger.error('A timeout occurred writing to location %s.'%(location))
            return False
        elif connector.status == False:
            return False
        elif connector.status == True:
            return True
    def get_size(self):
        if self.queue.qsize() != 0:
            return self.avg_data_size*self.queue.qsize()
        else:
            return 0
class DeliverFile(threading.Thread):
    def __init__(self,location=None,data=None,size=None,queue_size=None,queue_bytes='0',logger=None):
        threading.Thread.__init__(self)
        self.location=location
        self.data=data
        self.size=size
        self.queue_size=queue_size
        self.queue_bytes=queue_bytes
        self.logger=logger
        self.status=False
        self.daemon=True
        self.start()
    def run(self):
        start=time.time()
        try:
            self.location=self.location.rstrip('/')
            cmd=open(self.location,'a')
            if len(self.data) == 1:
                cmd.write(self.data[0]+'\n')
            else:
                cmd.write('\n'.join(self.data)+'\n')
            cmd.close()
            end=time.time()
            self.status=True
            self.logger.info("Data succesfully submitted to %s in %s seconds. %s commands processed. Delivery queue left %s items. Size: %s bytes"%(self.location,round(end-start,3),self.size,self.queue_size,self.queue_bytes))
        except Exception as error:
            self.status=False
            self.logger.error('Error submitting data to file %s. Reason: %s. Delivery queue left %s items. Size: %s bytes'%(self.location,error,self.queue_size,self.queue_bytes))
class DeliverNamedPipe(threading.Thread):
    def __init__(self,location=None,data=None,size=None,queue_size=None,queue_bytes='0',logger=None):
        threading.Thread.__init__(self)
        self.location=location
        self.data=data
        self.logger = logger
        self.status=False
        self.size=size
        self.queue_size=queue_size
        self.queue_bytes=queue_bytes
        self.daemon=True
        self.start()
    def run(self):
        start=time.time()
        try:
            if os.path.exists(self.location):
                mode = os.stat(self.location)[ST_MODE]
                if S_ISFIFO(mode):
                    cmd=open(self.location,'w')
                    if len(self.data) == 1:
                        cmd.write(self.data[0]+'\n')
                    else:
                        cmd.write('\n'.join(self.data)+'\n')
                    cmd.close()
                    end=time.time()
                    self.logger.info("Data succesfully submitted to %s in %s seconds. %s commands processed. Delivery queue left %s items. Size: %s bytes."%(self.location,round(end-start,3),self.size,self.queue_size,self.queue_bytes))
                    self.status=True
                else:
                    self.logger.error("%s is not a named pipe"%(self.location))
                    self.status=False
            else:
                self.logger.error("%s does not exist"%(self.location))
                self.status=False
        except Exception as error:
            self.logger.error("Error submitting data to %s. Reason: %s. Delivery queue left %s items. Size: %s bytes"%(self.location,error,self.queue_size,self.queue_bytes))
            self.status=False
class DeliverNscaweb(threading.Thread):
    def __init__(self,location=None,data=None,username=None,password=None,size=None,queue_size=None,queue_bytes='0',logger=None):
        threading.Thread.__init__(self)
        self.location=location
        self.data=data
        self.username=username
        self.password=password
        self.size=size
        self.queue_size=queue_size
        self.queue_bytes=queue_bytes
        self.logger=logger
        self.status=False
        self.opener = urllib2.build_opener()
        self.opener = urllib2.build_opener()
        self.opener.addheaders = [('User-agent', 'NSCAweb')]
        self.daemon=True
        self.start()
    def run(self):
        start=time.time()
        try:
            data = '\n'.join(self.data)
            data = data.decode("ascii", "ignore")
            nscaweb_data = urlencode({'username': self.username, 'password': self.password, 'input': data})
            conn = self.opener.open(self.location,nscaweb_data)
            conn.close()
            end=time.time()
            self.logger.info("Data succesfully submitted to %s in %s seconds. %s commands processed. Delivery queue left %s items. Size: %s bytes."%(self.location,round(end-start,3),self.size,self.queue_size,self.queue_bytes))
            self.status=True
        except Exception as error:
            self.logger.error('Error submitting data to NSCAweb %s. Reason: %s. Delivery queue left %s items. Size: %s bytes'%(self.location,error,self.queue_size,self.queue_bytes))
            self.status=False
class DeliverNrdp(threading.Thread):
    def __init__(self,location=None,data=None,username=None,password=None,token=None,size=None,queue_size=None,queue_bytes='0',logger=None):
        threading.Thread.__init__(self)
        self.location=location
        self.data=data
        self.username=username
        self.password=password
        self.token=token
        self.size=size
        self.queue_size=queue_size
        self.queue_bytes=queue_bytes
        self.logger=logger
        self.status=False
        self.opener = urllib2.build_opener()
        self.opener = urllib2.build_opener()
        self.opener.addheaders = [('User-agent', 'NSCAweb')]
        self.daemon=True
        self.start()
    def run(self):
        start=time.time()
        try:
            nrdp_data = urlencode({'token':self.token,'XMLDATA':'\n'.join(self.data),'cmd':'submitcheck','btnSubmit':'Submit Check Data'})
            conn = self.opener.open(self.location,nrdp_data)
            try:
                content = conn.read()
                conn.close()
                root = ElementTree.XML(content)

            except:
                self.logger.error('Error submitting data to NRDP %s. Reason: NRDP answer was non xml.'%(self.location))
                self.status=False
                conn.close()
                return

            if root[0].text == "-1":
                self.logger.error('Error submitting data to NRDP %s. Reason: %s.'%(self.location,root[1].text))
                self.status=False
                conn.close()
                return
            else:
                end=time.time()
                self.logger.info('Data succesfully submitted to %s in %s seconds. %s batch processed. Delivery queue left %s items. Size: %s bytes'%(self.location,round(end-start,3),"1",self.queue_size,self.queue_bytes))
                self.status=True
                conn.close()
                return
        except Exception as error:
            self.logger.error('Error submitting data to NRDP %s. Reason: %s. Delivery queue left %s items. Size: %s bytes'%(self.location,error,self.queue_size,self.queue_bytes))
            self.status=False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       authentication.py
#
#       Copyright 2010 Jelle Smet <web@smetj.net>
#
#       This file is part of Monitoring python library.
#
#           Monitoring python library is free software: you can redistribute it and/or modify
#           it under the terms of the GNU General Public License as published by
#           the Free Software Foundation, either version 3 of the License, or
#           (at your option) any later version.
#
#           Monitoring python library is distributed in the hope that it will be useful,
#           but WITHOUT ANY WARRANTY; without even the implied warranty of
#           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#           GNU General Public License for more details.
#
#           You should have received a copy of the GNU General Public License
#           along with Monitoring python library.  If not, see <http://www.gnu.org/licenses/>.
import hashlib
import pexpect

class Authenticate:
    '''Class providing authentication.'''
    def __init__(self,auth_type='default',database={},logger=None):
        self.auth_type=auth_type
        self.database=database
        self.logger=logger
        self.logger.info("Authentication method initiated of type: %s"%(self.auth_type))
    def do(self,username,password):
        '''Execute the authentication based upon type chosen at init.'''
        if self.auth_type == 'none':
            return True
        if username != None and password != None and username != '' and password != '':
            if self.auth_type == 'default':
                return self.__default(username=username,password=password)
            elif self.auth_type == 'pam':
                return self.__pam(username=username,password=password)
            else:
                self.logger.warn("Unknown authentication method.")
        else:
            self.logger.warn("Username or password can't be None or empty when authentication type is different than none.")
            return False
    def __default(self,username,password):
        '''Accepts username and password and depending upon the the auth style defined at init an auth type is chosen.'''
        if self.database.has_key(username):
            hash = hashlib.md5()
            hash.update(password+'\n')
            if hash.hexdigest() == self.database[username]:
                self.logger.info("Authentication succeeded for user %s."%(username))
                return True
            else:
                self.logger.warn("Authentication failed for user %s."%(username))
                return False
        else:
            self.logger.warn("User %s is not defined."%(username))
            return False
    def __pam(self, username, password):
        '''Accepts username and password and tried to use PAM for authentication'''
        child=None
        try:
            child = pexpect.spawn('/bin/su - %s'%(username))
            child.expect('Password:')
            child.sendline(password)
            #result=child.expect([username,'su: Authentication failure',pexpect.EOF])
            result=child.expect(username)
            child.close()
        except Exception as err:
            if child != None:
                child.close()
            self.logger.warn("Pam authentication failed.")
            return False
        if result == 0:
            self.logger.info("Authentication succeeded for user %s."%(username))
            return True
        else:
            self.logger.info("Authentication failed for user %s."%(username))
            return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       nscaweb
#
#       Copyright 2010 Jelle Smet <web@smetj.net>
#
#       This file is part of NSCAweb.
#
#           NSCAweb is free software: you can redistribute it and/or modify
#           it under the terms of the GNU General Public License as published by
#           the Free Software Foundation, either version 3 of the License, or
#           (at your option) any later version.
#
#           NSCAweb is distributed in the hope that it will be useful,
#           but WITHOUT ANY WARRANTY; without even the implied warranty of
#           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#           GNU General Public License for more details.
#
#           You should have received a copy of the GNU General Public License
#           along with NSCAweb.  If not, see <http://www.gnu.org/licenses/>.

import cherrypy
import os
import sys
import time
import signal
import Queue
import threading
import daemon
import select
import logging
from nscaweb.server import ThreadControl
from nscaweb.server import ConfigFileMonitor
from nscaweb.authentication import Authenticate
from nscaweb.communication import SubmitListener

from cherrypy import log
from configobj import ConfigObj
from optparse import OptionParser
from time import gmtime
from pkg_resources import get_distribution

class WebServer(threading.Thread):
    def __init__(self, host, port, pid, ssl="off", ssl_certificate=None, ssl_private_key=None, htmlContent=None, enable_logging="1", blockcallback=None):
        threading.Thread.__init__(self)
        self.host=host
        self.port=port
        self.ssl=ssl
        self.ssl_certificate=ssl_certificate
        self.ssl_private_key=ssl_private_key
        self.htmlContent=htmlContent
        self.enable_logging=enable_logging
        self.loop=blockcallback
        self.daemon=True
        self.start()

    def run(self):
        logger.info("WebServer thread started on %s:%s"%(self.host,self.port))
        config={'global': {'server.socket_host': self.host}}
        cherrypy.config.update(
                                {
                                'global':
                                    {
                                    'server.socket_port': int(self.port),
                                    'server.socket_host': self.host,
                                    'tools.sessions.on': False,
                                    'log.screen': False
                                    }
                                }
                                )

        #check if we need to run over https or not
        if self.ssl == "on":
            cherrypy.config.update(
                                    {
                                    'server.ssl_certificate': self.ssl_certificate,
                                    'server.ssl_private_key': self.ssl_private_key
                                    }
                                  )

        cp_app = cherrypy.tree.mount(self.htmlContent,'/',config=config)

        if self.enable_logging == '1':
            cp_app.log.access_log_format = '%(h)s %(l)s %(u)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"'
            cp_app.log.access_log = logger
            #log.error_log = logger
            #help(log.error_log)
        cherrypy.engine.start()
        while self.loop.block() == True:
            time.sleep(0.1)
        cherrypy.engine.exit()
        logger.info("WebServer thread stopped.")

class HtmlContent():
    '''Class which provides form to accept incoming NSCAweb data.'''

    def __init__(self,queueDefinitions,submitListener,authentication):
        self.queueDefinitions=queueDefinitions
        self.submitListener=submitListener
        self.authentication=authentication

    def index(self,*args,**form):
        pass
    index.exposed = False

    def queue(self,*args,**form):
        try:
            if form.has_key('username') and form.has_key('password') and form.has_key('input'):
                if self.authentication.do(username=form['username'],password=form['password']):
                    form['input']=form['input'].rstrip()
                    if len(args)==0:
                        counter=0
                        for line in form['input'].split('\n'):
                            counter+=1
                            #Since this is the broadcast, run over all queues and dump the line in each queue
                            for name in self.queueDefinitions:
                                if self.queueDefinitions[name]['enable'] == "1":
                                    package = self.construct_package(name=name,queueDefinitions=self.queueDefinitions,line=line)
                                    try:
                                        self.submitListener.dump(package)
                                    except Exception:
                                        logger.warn("Queue size limit reached. Data purged.")
                                        continue
                        logger.info("%s items dumped into the broadcast queue for user %s from IP %s."%(counter,form['username'],cherrypy.request.remote.ip))
                    elif len(args)==1:
                        if self.queueDefinitions.has_key(args[0]) and self.queueDefinitions[args[0]]['enable'] == "1":
                            counter=0
                            for line in form['input'].split('\n'):
                                counter+=1
                                package = self.construct_package(name=args[0],queueDefinitions=self.queueDefinitions,line=line)
                                try:
                                    self.submitListener.dump(package)
                                except Exception:
                                    logger.warn("Queue %s size limit reached. Data discarted."%(args[0]))
                                    raise cherrypy.HTTPError("500 Internal Server Error", "Queue %s is full. Data discarted."%(args[0]))

                            logger.info("%s items dumped into queue %s for user %s from IP %s."%(counter,args[0],form['username'],cherrypy.request.remote.ip))
                        else:
                            logger.warn("Queue %s does not exist or is disabled. Data is purged for user %s from IP %s."%(args[0],form['username'],cherrypy.request.remote.ip))
                    else:
                        logger.warn("Malformed url for user %s from IP %s"%(form['username'],cherrypy.request.remote.ip))
                else:
                    logger.warn("Access denied for user %s from IP %s."%(form['username'],cherrypy.request.remote.ip))
            else:
                logger.warn("Incomplete data submitted from IP %s."%(cherrypy.request.remote.ip))
        except cherrypy.HTTPError as err:
            raise cherrypy.HTTPError("500 Internal Server Error", str(err[1]))
        except Exception as error:
            logger.warn("Malformed data submitted from IP %s. Maybe something wrong with the destination configuration in the config file."%(cherrypy.request.remote.ip))
    queue.exposed = True

    def default(self, *args, **kwargs):
        pass
    default.exposed = True

    def construct_package(self,name,queueDefinitions,line):
        package={'destination':{
            'type' : queueDefinitions[name]['type'],
            'locations' : queueDefinitions[name]['locations'].split(','),
            'username' : queueDefinitions[name].get('username',None),
            'password' : queueDefinitions[name].get('password',None),
            'token' : queueDefinitions[name].get('token',None)
            },
            'external_command' : line
            }
        return package

class Logger():
    '''Creates a logger class.'''

    def __init__(self,logfile='', scrlog=True, syslog='1', loglevel=logging.INFO):
        self.logfile=logfile
        self.scrlog=scrlog
        self.syslog=syslog
        self.loglevel=loglevel
        self.format=logging.Formatter(fmt='%(asctime)s %(levelname)s: %(message)s',datefmt=None)
        self.syslog_format=logging.Formatter(fmt='NSCAweb: %(message)s',datefmt=None)
        self.log = logging.getLogger(__name__)
        self.log.setLevel(self.loglevel)
        if logfile != '':
            self.file_handler = logging.FileHandler( logfile )
            self.file_handler.setFormatter(self.format)
            self.log.addHandler(self.file_handler)
        if scrlog == True:
            self.scr_handler = logging.StreamHandler()
            self.scr_handler.setFormatter(self.format)
            self.log.addHandler(self.scr_handler)
        if syslog == '1':
            from logging.handlers import SysLogHandler
            if os.path.exists('/var/run/syslog'):
                self.sys_handler = SysLogHandler(address='/var/run/syslog')
            else:
                self.sys_handler = SysLogHandler(address='/dev/log')

            self.sys_handler.setFormatter(self.syslog_format)
            self.log.addHandler(self.sys_handler)

class NamedPipe(threading.Thread):

    def __init__(self, directory, name, submitListener, queueDefinitions=None, blockcallback=None):
        threading.Thread.__init__(self)
        self.directory=directory
        self.name=name
        self.submitListener=submitListener
        self.queueDefinitions=queueDefinitions
        self.loop=blockcallback
        self.absolutePath=self.directory+'/'+self.name
        self.daemon=True
        self.start()

    def run(self):
        try:
            os.unlink ( self.absolutePath )
        except:
            pass
        try:
            os.mkfifo ( self.absolutePath )
        except Exception as err:
            logger.critical('There was an error creating the named pipe %s. Reason: %s'%(self.absolutePath,err))
            return
        else:
            logger.info('Named pipe listener created with name %s'%(self.name))
        try:
            fifo = open(self.absolutePath,'r')
            while self.loop.block() == True:
                line = fifo.readline()[:-1]
                if len(line) > 0:
                    if self.name == 'broadcast':
                        for queueDefinition in self.queueDefinitions:
                            if self.queueDefinitions[queueDefinition]['enable'] == '1':
                                package = self.construct_package(name=queueDefinition,queueDefinitions=self.queueDefinitions,line=line)
                                self.submitListener.input_queue.put(package)
                    else:
                        package = self.construct_package(name=self.name,queueDefinitions=self.queueDefinitions,line=line)
                        self.submitListener.input_queue.put(package)
                else:
                    time.sleep(0.1)
        except Exception as err:
            logger.critical('There was an error reading from the named pipe %s. Reason: %s'%(self.absolutePath,err))
            os.unlink(self.absolutePath)
            return
        try:
            fifo.close()
            os.unlink ( self.absolutePath )
        except:
            pass
        logger.info('Named pipe listener with name %s has exit.'%(self.name))
        return

    def stop(self):
        try:
            breakBlock = open (self.absolutePath,'r+')
            breakBlock.write('\n')
            breakBlock.close()
        except:
            pass

    def construct_package(self, name, queueDefinitions, line):
        package={'destination':{
            'type' : queueDefinitions[name]['type'],
            'locations' : queueDefinitions[name]['locations'].split(','),
            'username' : queueDefinitions[name].get('username',None),
            'password' : queueDefinitions[name].get('password',None),
            'token' : queueDefinitions[name].get('token',None)
            },
            'external_command' : line
            }
        return package

class Server():
    def __init__(self,configfile=None):
        self.configfile=configfile
        try:
            self.config=ConfigObj(self.configfile)
        except Exception as err:
            sys.stderr.write('There appears to be an error in your configfile:\n')
            sys.stderr.write('\t'+ str(type(err))+" "+str(err) + "\n" )
            os.kill(os.getpid(),signal.SIGKILL)
    def check_running(self):
        if (os.path.exists(self.config['application']['pidfile'])):
            pid_file = open(self.config['application']['pidfile'],'r')
            pid = pid_file.readline()
            pid_file.close()
            try:
                os.kill(int(pid),0)
            except:
                sys.stderr.write('PID file detected but no process with such a PID.  I will overwrite it.\n')
            else:
                sys.stderr.write('There is already a version of NSCAweb running with pid %s\n'%(pid))
                sys.exit(1)
    def start(self,debug=False):

        #Set logging environment
        global logger
        logger_object = Logger( logfile=self.config['logging'].get('logfile',''),
                    scrlog=debug,
                    syslog=self.config['logging'].get('enable_syslog',"0")
                    )
        logger = logger_object.log
        logger.info('started')


        #Create pid
        pidfile=open(self.config["application"]["pidfile"],'w')
        pidfile.write(str(os.getpid()))
        pidfile.close()

        #Create home for threads
        server=ThreadControl()

        #Start a ConfigFileMonitor thread
        server.threads['configfilemonitor'] = ConfigFileMonitor (file=self.configfile,
                                        logger=logger,
                                        blockcallback=server)

        #Create Authentication object
        auth = Authenticate (   auth_type='default',
                        database=server.threads['configfilemonitor'].file['authentication'],
                        logger=logger)

        #Create SubmitListener object
        server.threads['submitListener'] = SubmitListener(  quota=int(server.threads['configfilemonitor'].file['application'].get('queue_quota','0')),
                                    logger=logger,
                                    blockcallback=server)

        #Create an HtmlContent object
        htmlContent = HtmlContent ( queueDefinitions=server.threads['configfilemonitor'].file['destinations'],
                        submitListener=server.threads['submitListener'],
                        authentication = auth)

        #Start the WebServer thread
        server.threads['webserver'] = WebServer (host=server.threads['configfilemonitor'].file['application']['host'],
                            port=server.threads['configfilemonitor'].file['application']['port'],
                            pid=server.threads['configfilemonitor'].file['application']['pidfile'],
                            ssl=server.threads['configfilemonitor'].file['application']['sslengine'],
                            ssl_certificate=server.threads['configfilemonitor'].file['application']['sslcertificate'],
                            ssl_private_key=server.threads['configfilemonitor'].file['application']['sslprivatekey'],
                            htmlContent=htmlContent,
                            enable_logging=self.config['logging'].get('enable_http_logging',"0"),
                            blockcallback=server)

        #Start NamedPipe threads
        if server.threads['configfilemonitor'].file['pipes']['enable'] == "1":
            for destination in server.threads['configfilemonitor'].file['destinations']:
                if server.threads['configfilemonitor'].file['destinations'][destination]['enable'] == "1":
                    server.threads['namedPipe'+destination]  = NamedPipe(   directory=server.threads['configfilemonitor'].file['pipes']['directory'],
                                                                            name=destination,
                                                                            submitListener=server.threads['submitListener'],
                                                                            queueDefinitions=server.threads['configfilemonitor'].file['destinations'],
                                                                            blockcallback=server)

            server.threads['namedPipe'+destination]  = NamedPipe(   directory=server.threads['configfilemonitor'].file['pipes']['directory'],
                                                                    name='broadcast',
                                                                    submitListener=server.threads['submitListener'],
                                                                    queueDefinitions=server.threads['configfilemonitor'].file['destinations'],
                                                                    blockcallback=server)
        #Block here
        try:
            while server.block()==True:
                time.sleep(1)
        except KeyboardInterrupt:
            print ("Stopping all queues in a polite way. Press ctrl+c again to force stop.")
            try:
                #write bogus data into pipes
                while server.stop_all([server.threads,server.queues]) == False:
                    time.sleep(1)
                    logger.info('Waiting for all queues to end.')
            except KeyboardInterrupt:
                os.remove(server.threads['configfilemonitor'].file['application']['pidfile'])
                exit
            try:
                os.remove(server.threads['configfilemonitor'].file['application']['pidfile'])
            except:
                pass

    def stop(self):
        try:
            pidfile=open(self.config["application"]["pidfile"],'r')
            pid=pidfile.read()
            pidfile.close()
            os.kill(int(pid),signal.SIGINT)
        except Exception as error:
            sys.stderr.write('Could not stop NSCAweb.  Reason: %s\n'%error)

    def kill(self):
        try:
            pidfile=open(self.config["application"]["pidfile"],'r')
            pid=pidfile.read()
            pidfile.close()
            os.remove(self.config["application"]["pidfile"])
            os.kill(int(pid),signal.SIGKILL)
        except Exception as error:
            sys.stderr.write('Could not kill NSCAweb.  Reason: %s\n'%error)

class Help():
    def __init__(self):
        pass
    def usage(self):
        print ('NSCAweb %s Copyright 2009,2010,2011 by Jelle Smet <jelle@smetj.net>' %( get_distribution('nscaweb').version))
        print ('''Usage: nscaweb command --config configfile

    Valid commands:

        start   Starts the nscaweb daemon in the background.

        stop    Gracefully stops the nscaweb daemon running in the background.

        kill    Kills the nscaweb daemon running with the pid defined in your config file.

        debug   Starts the nscaweb daemon in the foreground while showing real time log and debug messages.
            The process can be stopped with ctrl+c which will ends NSCAweb gracefully.
            A second ctrl+c will kill NSCAweb.

    Parameters:
        --config    Defines the location of the config file to use.  The parameter is obligatory.
                -c can also be used.

NSCAweb is distributed under the Terms of the GNU General Public License Version 3. (http://www.gnu.org/licenses/gpl-3.0.html)

For more information please visit http://www.smetj.net/nscaweb/
''')
        return()

def main():

    try:
        #Parse command line options
        parser = OptionParser()
        parser.add_option("-c",  "--config", dest="configfile", default="/opt/nscaweb/etc/main.conf", type="string", help="config file")
        (commandline_options,commandline_actions)=parser.parse_args()
        server=Server(configfile=commandline_options.configfile)
        #Execute command
        if len(commandline_actions) != 1:
            Help().usage()
            sys.exit
        elif commandline_actions[0] == 'start':
            server.check_running()
            print ("Starting NSCAweb in background.")
            with daemon.DaemonContext():
                server.start()
        elif commandline_actions[0] == 'debug':
            print ("Starting NSCAweb in foreground.")
            server.check_running()
            server.start(debug=True)
        elif commandline_actions[0] == 'stop':
            print ("Stopping NSCAweb gracefully.  Tail log for progress.")
            server.stop()
        elif commandline_actions[0] == 'kill':
            print ("Killing NSCAweb forcefully.")
            server.kill()
        elif commandline_actions[0] == 'dump':
            pass
        else:
            Help().usage()
            print ('Unknown option %s \n' %(commandline_actions[0]))
            sys.exit()
    except Exception as err:
        sys.stderr.write('A fatal error has occurred.\n')
        sys.stderr.write('Please file a bug report to https://github.com/smetj/nscaweb/issues including:\n')
        sys.stderr.write('\t - NSCAweb version.\n')
        sys.stderr.write('\t - A copy of your config file.\n')
        sys.stderr.write('\t - Your OS and version.\n')
        sys.stderr.write('\t - Your Python and Cherrypy version.\n')
        sys.stderr.write('\t - The steps to take to reproduce this error.\n')
        sys.stderr.write('\t - This piece of information: '+ str(type(err))+" "+str(err) + "\n" )
        sys.exit(1)

if __name__ == '__main__':
    main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       server.py
#
#       Copyright 2010 Jelle Smet <web@smetj.net>
#
#       This file is part of Monitoring python library.
#
#           Monitoring python library is free software: you can redistribute it and/or modify
#           it under the terms of the GNU General Public License as published by
#           the Free Software Foundation, either version 3 of the License, or
#           (at your option) any later version.
#
#           Monitoring python library is distributed in the hope that it will be useful,
#           but WITHOUT ANY WARRANTY; without even the implied warranty of
#           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#           GNU General Public License for more details.
#
#           You should have received a copy of the GNU General Public License
#           along with Monitoring python library.  If not, see <http://www.gnu.org/licenses/>.
import threading
import Queue
import os
import sys
import signal
import time
from time import gmtime
from configobj import ConfigObj

class ThreadControl():
    def __init__(self):
        self.ignition_key=True
        self.last_ignition_key=True
        self.threads={}
        self.queues={}

    def key(self):
        return self.ignition_key

    def block(self):
        return self.ignition_key

    def last_block(self):
        return self.last_ignition_key

    def stop_all(self,list):
        self.ignition_key=False
        for group in list:
            for thread in group:
                try:
                    group[thread].stop()
                except:
                    pass
                if group[thread].loop and group[thread].name != 'last' and group[thread].is_alive():
                    return False
        self.last_ignition_key=False
        time.sleep(1)


class ConfigFileMonitor(threading.Thread):
    '''Monitors if the configfile has changed and if so, reads it.'''

    def __init__(self,file,logger,blockcallback):
        threading.Thread.__init__(self)
        self.config_file=file
        self.logger=logger
        self.loop=blockcallback

        try:
            self.file=ConfigObj(self.config_file)
        except Exception as err:
            sys.stderr.write('There appears to be an error in your configfile:\n')
            sys.stderr.write('\t'+ str(type(err))+" "+str(err) + "\n" )
            os.kill(os.getpid(),signal.SIGKILL)

        self.file_stats     =os.stat(self.config_file)
        self.current_stats  =os.stat(self.config_file)
        self.daemon=True
        self.start()

    def run(self):
        self.logger.info("ConfigFileMonitor thread started.")
        while self.loop.block()==True:
            self.current_stats  =os.stat(self.config_file)
            if (self.current_stats.st_ctime != self.file_stats.st_ctime):
                try:
                    self.file_stats=self.current_stats
                    self.logger.info("The config file has changed.")
                    self.config=ConfigObj(self.config_file)
                    self.logger.info("The config file is reloaded.")
                except:
                    self.logger.error("The config file has not been loaded as it contains errors.")
                    previous_value=current_value
            time.sleep(1)
        self.logger.info("ConfigFileMonitor thread stopped.")
